@@ -4,7 +4,7 @@
 
     Global creation environment.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -16,6 +16,7 @@ from copy import copy
 from io import BytesIO
 from os import path
 from typing import Any, Callable, Dict, Generator, IO, Iterator, List, Set, Tuple, Union
+from typing import cast
 
 from docutils import nodes
 from docutils.nodes import Node
@@ -43,6 +44,7 @@ if False:
     from sphinx.application import Sphinx
     from sphinx.builders import Builder
 
+
 logger = logging.getLogger(__name__)
 
 default_settings = {
@@ -169,11 +171,6 @@ class BuildEnvironment:
         self.domaindata = {}        # type: Dict[str, Dict]
                                     # domainname -> domain-specific dict
 
-        # Other inventories
-        self.indexentries = {}      # type: Dict[str, List[Tuple[str, str, str, str, str]]]
-                                    # docname -> list of
-                                    # (type, str, target, aliasname)
-
         # these map absolute path -> (docnames, unique filename)
         self.images = FilenameUniqDict()    # type: FilenameUniqDict
         self.dlfiles = DownloadFiles()      # type: DownloadFiles
@@ -750,6 +747,14 @@ class BuildEnvironment:
         node.line = lineno
         self.get_domain('changeset').note_changeset(node)  # type: ignore
 
+    @property
+    def indexentries(self) -> Dict[str, List[Tuple[str, str, str, str, str]]]:
+        warnings.warn('env.indexentries() is deprecated. Please use IndexDomain instead.',
+                      RemovedInSphinx40Warning, stacklevel=2)
+        from sphinx.domains.index import IndexDomain
+        domain = cast(IndexDomain, self.get_domain('index'))
+        return domain.entries
+
 
 from sphinx.errors import NoUri  # NOQA
 
